"""SPI Class for RP2040"""
from machine import SPI as _SPI
from machine import Pin
from microcontroller.pin import spiPorts

# pylint: disable=protected-access, no-self-use
class SPI:
    """Custom SPI Class for RP2040"""

    MSB = _SPI.MSB

    def __init__(self, clock, MOSI=None, MISO=None, *, baudrate=1000000):
        self._frequency = baudrate
        for portId, portSck, portMosi, portMiso in spiPorts:
            if (
                (clock == portSck)
                and MOSI in (portMosi, None)  # Clock is required!
                and MISO in (portMiso, None)  # But can do with just output
            ):  # Or just input
                mosiPin = Pin(portMosi.id) if MOSI else None
                misoPin = Pin(portMiso.id) if MISO else None
                self._spi = _SPI(
                    portId,
                    sck=Pin(portSck.id),
                    mosi=mosiPin,
                    miso=misoPin,
                    baudrate=baudrate,
                )
                break
        else:
            raise ValueError(
                "No Hardware SPI on (SCLK, MOSI, MISO)={}\nValid SPI ports:{}".format(
                    (clock, MOSI, MISO), spiPorts
                )
            )

    # pylint: disable=too-many-arguments,unused-argument
    def init(
        self,
        baudrate=1000000,
        polarity=0,
        phase=0,
        bits=8,
        firstbit=_SPI.MSB,
        sck=None,
        mosi=None,
        miso=None,
    ):
        """Initialize the Port"""
        self._frequency = baudrate
        self._spi.init(
            baudrate=baudrate,
            polarity=polarity,
            phase=phase,
            bits=bits,
            firstbit=firstbit,
        )

    # pylint: enable=too-many-arguments

    @property
    def frequency(self):
        """Return the current frequency"""
        return self._frequency

    def write(self, buf, start=0, end=None):
        """Write data from the buffer to SPI"""
        self._spi.write(buf)

    def readinto(self, buf, start=0, end=None, write_value=0):
        """Read data from SPI and into the buffer"""
        self._spi.readinto(buf)

    # pylint: disable=too-many-arguments
    def write_readinto(
        self, buffer_out, buffer_in, out_start=0, out_end=None, in_start=0, in_end=None
    ):
        """Perform a half-duplex write from buffer_out and then
        read data into buffer_in
        """
        self._spi.write_readinto(
            buffer_out,
            buffer_in,
            out_start=out_start,
            out_end=out_end,
            in_start=in_start,
            in_end=in_end,
        )

    # pylint: enable=too-many-arguments



"""Pin definitions for the FTDI FT2232H"""
from adafruit_blinka.microcontroller.ftdi_mpsse.ft2232h import pin

# MPSSE A
AD4 = pin.AD4
AD5 = pin.AD5
AD6 = pin.AD6
AD7 = pin.AD7
AC0 = pin.AC0
AC1 = pin.AC1
AC2 = pin.AC2
AC3 = pin.AC3
AC4 = pin.AC4
AC5 = pin.AC5
AC6 = pin.AC6
AC7 = pin.AC7

SDA0 = pin.SDA0
SCL0 = pin.SCL0

SCK0 = pin.SCK0
SCLK0 = pin.SCLK0
MOSI0 = pin.MOSI0
MISO0 = pin.MISO0

# MPSSE B
BD4 = pin.BD4
BD5 = pin.BD5
BD6 = pin.BD6
BD7 = pin.BD7
BC0 = pin.BC0
BC1 = pin.BC1
BC2 = pin.BC2
BC3 = pin.BC3
BC4 = pin.BC4
BC5 = pin.BC5
BC6 = pin.BC6
BC7 = pin.BC7

SDA1 = pin.SDA1
SCL1 = pin.SCL1

SCK1 = pin.SCK1
SCLK1 = pin.SCLK1
MOSI1 = pin.MOSI1
MISO1 = pin.MISO1



"""
`keypad` - Support for scanning keys and key matrices
===========================================================
See `CircuitPython:keypad` in CircuitPython for more details.

* Author(s): Melissa LeBlanc-Williams
"""

import time
import threading
from collections import deque
import digitalio


class Event:
    """A key transition event."""

    def __init__(self, key_number=0, pressed=True):
        """
        Create a key transition event, which reports a key-pressed or key-released transition.

        :param int key_number: the key number
        :param bool pressed: ``True`` if the key was pressed; ``False`` if it was released.
        """
        self._key_number = key_number
        self._pressed = pressed

    @property
    def key_number(self):
        """The key number."""
        return self._key_number

    @property
    def pressed(self):
        """
        ``True`` if the event represents a key down (pressed) transition.
        The opposite of `released`.
        """
        return self._pressed

    @property
    def released(self):
        """
        ``True`` if the event represents a key up (released) transition.
        The opposite of `pressed`.
        """
        return not self._pressed

    def __eq__(self, other):
        """
        Two `Event` objects are equal if their `key_number`
        and `pressed`/`released` values are equal.
        """
        return self.key_number == other.key_number and self.pressed == other.pressed

    def __hash__(self):
        """Returns a hash for the `Event`, so it can be used in dictionaries, etc.."""
        return hash(self._key_number)

    def __repr__(self):
        """Return a textual representation of the object"""
        return "<Event: key_number {} {}>".format(
            self.key_number, "pressed" if self._pressed else "released"
        )


class _EventQueue:
    """
    A queue of `Event` objects, filled by a `keypad` scanner such as `Keys` or `KeyMatrix`.

    You cannot create an instance of `_EventQueue` directly. Each scanner creates an
    instance when it is created.
    """

    def __init__(self, max_events):
        self._events = deque([], max_events)
        self._overflowed = False

    def get(self):
        """
        Return the next key transition event. Return ``None`` if no events are pending.

        Note that the queue size is limited; see ``max_events`` in the constructor of
        a scanner such as `Keys` or `KeyMatrix`.
        If a new event arrives when the queue is full, the event is discarded, and
        `overflowed` is set to ``True``.

        :return: the next queued key transition `Event`
        :rtype: Optional[Event]
        """
        if not self._events:
            return None
        return self._events.popleft()

    def get_into(self, event):
        """Store the next key transition event in the supplied event, if available,
        and return ``True``.
        If there are no queued events, do not touch ``event`` and return ``False``.

        The advantage of this method over ``get()`` is that it does not allocate storage.
        Instead you can reuse an existing ``Event`` object.

        Note that the queue size is limited; see ``max_events`` in the constructor of
        a scanner such as `Keys` or `KeyMatrix`.

        :return ``True`` if an event was available and stored, ``False`` if not.
        :rtype: bool
        """
        if not self._events:
            return False
        next_event = self._events.popleft()
        # pylint: disable=protected-access
        event._key_number = next_event._key_number
        event._pressed = next_event._pressed
        # pylint: enable=protected-access
        return True

    def clear(self):
        """
        Clear any queued key transition events. Also sets `overflowed` to ``False``.
        """
        self._events.clear()
        self._overflowed = False

    def __bool__(self):
        """``True`` if `len()` is greater than zero.
        This is an easy way to check if the queue is empty.
        """
        return len(self._events) > 0

    def __len__(self):
        """Return the number of events currently in the queue. Used to implement ``len()``."""
        return len(self._events)

    @property
    def overflowed(self):
        """
        ``True`` if an event could not be added to the event queue because it was full. (read-only)
        Set to ``False`` by  `clear()`.
        """
        return self._overflowed

    def keypad_eventqueue_record(self, key_number, current):
        """Record a new event"""
        if len(self._events) == self._events.maxlen:
            self._overflowed = True
        else:
            self._events.append(Event(key_number, current))


class _KeysBase:
    def __init__(self, interval, max_events, scanning_function):
        self._interval = interval
        self._last_scan = time.monotonic()
        self._events = _EventQueue(max_events)
        self._scanning_function = scanning_function
        self._scan_thread = threading.Thread(target=self._scanning_loop, daemon=True)
        self._scan_thread.start()

    @property
    def events(self):
        """The EventQueue associated with this Keys object. (read-only)"""
        return self._events

    def deinit(self):
        """Stop scanning"""
        if self._scan_thread.is_alive():
            self._scan_thread.join()

    def __enter__(self):
        """No-op used by Context Managers."""
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        """
        Automatically deinitializes when exiting a context. See
        :ref:`lifetime-and-contextmanagers` for more info.
        """
        self.deinit()

    def _scanning_loop(self):
        while True:
            remaining_delay = self._interval - (time.monotonic() - self._last_scan)
            if remaining_delay > 0:
                time.sleep(remaining_delay)
            self._last_scan = time.monotonic()
            self._scanning_function()


class Keys(_KeysBase):
    """Manage a set of independent keys."""

    def __init__(
        self, pins, *, value_when_pressed, pull=True, interval=0.02, max_events=64
    ):
        """
        Create a `Keys` object that will scan keys attached to the given sequence of pins.
        Each key is independent and attached to its own pin.

        An `EventQueue` is created when this object is created and is available in the
        `events` attribute.

        :param Sequence[microcontroller.Pin] pins: The pins attached to the keys.
          The key numbers correspond to indices into this sequence.
        :param bool value_when_pressed: ``True`` if the pin reads high when the key is pressed.
          ``False`` if the pin reads low (is grounded) when the key is pressed.
          All the pins must be connected in the same way.
        :param bool pull: ``True`` if an internal pull-up or pull-down should be
          enabled on each pin. A pull-up will be used if ``value_when_pressed`` is ``False``;
          a pull-down will be used if it is ``True``.
          If an external pull is already provided for all the pins, you can set
          ``pull`` to ``False``.
          However, enabling an internal pull when an external one is already present is not
          a problem;
          it simply uses slightly more current.
        :param float interval: Scan keys no more often than ``interval`` to allow for debouncing.
          ``interval`` is in float seconds. The default is 0.020 (20 msecs).
        :param int max_events: maximum size of `events` `EventQueue`:
          maximum number of key transition events that are saved.
          Must be >= 1.
          If a new event arrives when the queue is full, the oldest event is discarded.
        """
        self._digitalinouts = []
        for pin in pins:
            dio = digitalio.DigitalInOut(pin)
            if pull:
                dio.pull = (
                    digitalio.Pull.DOWN if value_when_pressed else digitalio.Pull.UP
                )
            self._digitalinouts.append(dio)

        self._currently_pressed = [False] * len(pins)
        self._previously_pressed = [False] * len(pins)
        self._value_when_pressed = value_when_pressed

        super().__init__(interval, max_events, self._keypad_keys_scan)

    def deinit(self):
        """Stop scanning and release the pins."""
        super().deinit()
        for dio in self._digitalinouts:
            dio.deinit()

    def reset(self):
        """Reset the internal state of the scanner to assume that all keys are now released.
        Any key that is already pressed at the time of this call will therefore immediately cause
        a new key-pressed event to occur.
        """
        self._currently_pressed = self._previously_pressed = [False] * self.key_count

    @property
    def key_count(self):
        """The number of keys that are being scanned. (read-only)"""
        return len(self._digitalinouts)

    def _keypad_keys_scan(self):
        for key_number, dio in enumerate(self._digitalinouts):
            self._previously_pressed[key_number] = self._currently_pressed[key_number]
            current = dio.value == self._value_when_pressed
            self._currently_pressed[key_number] = current
            if self._previously_pressed[key_number] != current:
                self._events.keypad_eventqueue_record(key_number, current)


class KeyMatrix(_KeysBase):
    """Manage a 2D matrix of keys with row and column pins."""

    # pylint: disable=too-many-arguments
    def __init__(
        self,
        row_pins,
        column_pins,
        columns_to_anodes=True,
        interval=0.02,
        max_events=64,
    ):
        """
        Create a `Keys` object that will scan the key matrix attached to the given row and
        column pins.
        There should not be any external pull-ups or pull-downs on the matrix:
        ``KeyMatrix`` enables internal pull-ups or pull-downs on the pins as necessary.

        The keys are numbered sequentially from zero. A key number can be computed
        by ``row * len(column_pins) + column``.

        An `EventQueue` is created when this object is created and is available in the `events`
        attribute.

        :param Sequence[microcontroller.Pin] row_pins: The pins attached to the rows.
        :param Sequence[microcontroller.Pin] column_pins: The pins attached to the colums.
        :param bool columns_to_anodes: Default ``True``.
          If the matrix uses diodes, the diode anodes are typically connected to the column pins,
          and the cathodes should be connected to the row pins. If your diodes are reversed,
          set ``columns_to_anodes`` to ``False``.
        :param float interval: Scan keys no more often than ``interval`` to allow for debouncing.
          ``interval`` is in float seconds. The default is 0.020 (20 msecs).
        :param int max_events: maximum size of `events` `EventQueue`:
          maximum number of key transition events that are saved.
          Must be >= 1.
          If a new event arrives when the queue is full, the oldest event is discarded.
        """
        self._row_digitalinouts = []
        for row_pin in row_pins:
            row_dio = digitalio.DigitalInOut(row_pin)
            row_dio.switch_to_input(
                pull=(digitalio.Pull.UP if columns_to_anodes else digitalio.Pull.DOWN)
            )
            self._row_digitalinouts.append(row_dio)

        self._column_digitalinouts = []
        for column_pin in column_pins:
            col_dio = digitalio.DigitalInOut(column_pin)
            col_dio.switch_to_input(
                pull=(digitalio.Pull.UP if columns_to_anodes else digitalio.Pull.DOWN)
            )
            self._column_digitalinouts.append(col_dio)
        self._currently_pressed = [False] * len(column_pins) * len(row_pins)
        self._previously_pressed = [False] * len(column_pins) * len(row_pins)
        self._columns_to_anodes = columns_to_anodes

        super().__init__(interval, max_events, self._keypad_keymatrix_scan)

    # pylint: enable=too-many-arguments

    @property
    def key_count(self):
        """The number of keys that are being scanned. (read-only)"""
        return len(self._row_digitalinouts) * len(self._column_digitalinouts)

    def deinit(self):
        """Stop scanning and release the pins."""
        super().deinit()
        for row_dio in self._row_digitalinouts:
            row_dio.deinit()
        for col_dio in self._column_digitalinouts:
            col_dio.deinit()

    def reset(self):
        """
        Reset the internal state of the scanner to assume that all keys are now released.
        Any key that is already pressed at the time of this call will therefore immediately cause
        a new key-pressed event to occur.
        """
        self._previously_pressed = self._currently_pressed = [False] * self.key_count

    def _row_column_to_key_number(self, row, column):
        return row * len(self._column_digitalinouts) + column

    def _keypad_keymatrix_scan(self):
        for row, row_dio in enumerate(self._row_digitalinouts):
            row_dio.switch_to_output(
                value=(not self._columns_to_anodes),
                drive_mode=digitalio.DriveMode.PUSH_PULL,
            )
            for col, col_dio in enumerate(self._column_digitalinouts):
                key_number = self._row_column_to_key_number(row, col)
                self._previously_pressed[key_number] = self._currently_pressed[
                    key_number
                ]
                current = col_dio.value != self._columns_to_anodes
                self._currently_pressed[key_number] = current
                if self._previously_pressed[key_number] != current:
                    self._events.keypad_eventqueue_record(key_number, current)
            row_dio.value = self._columns_to_anodes
            row_dio.switch_to_input(
                pull=(
                    digitalio.Pull.UP
                    if self._columns_to_anodes
                    else digitalio.Pull.DOWN
                )
            )


class ShiftRegisterKeys(_KeysBase):
    """Manage a set of keys attached to an incoming shift register."""

    def __init__(
        self,
        *,
        clock,
        data,
        latch,
        value_to_latch=True,
        key_count,
        value_when_pressed,
        interval=0.02,
        max_events=64,
    ):
        """
        Create a `Keys` object that will scan keys attached to a parallel-in serial-out
        shift register like the 74HC165 or CD4021.
        Note that you may chain shift registers to load in as many values as you need.

        Key number 0 is the first (or more properly, the zero-th) bit read. In the
        74HC165, this bit is labeled ``Q7``. Key number 1 will be the value of ``Q6``, etc.

        An `EventQueue` is created when this object is created and is available in the
        `events` attribute.

        :param microcontroller.Pin clock: The shift register clock pin.
          The shift register should clock on a low-to-high transition.
        :param microcontroller.Pin data: the incoming shift register data pin
        :param microcontroller.Pin latch:
          Pin used to latch parallel data going into the shift register.
        :param bool value_to_latch: Pin state to latch data being read.
          ``True`` if the data is latched when ``latch`` goes high
          ``False`` if the data is latched when ``latch goes low.
          The default is ``True``, which is how the 74HC165 operates. The CD4021 latch is
          the opposite. Once the data is latched, it will be shifted out by toggling the
          clock pin.
        :param int key_count: number of data lines to clock in
        :param bool value_when_pressed: ``True`` if the pin reads high when the key is pressed.
          ``False`` if the pin reads low (is grounded) when the key is pressed.
        :param float interval: Scan keys no more often than ``interval`` to allow for debouncing.
          ``interval`` is in float seconds. The default is 0.020 (20 msecs).
        :param int max_events: maximum size of `events` `EventQueue`:
          maximum number of key transition events that are saved.
          Must be >= 1.
          If a new event arrives when the queue is full, the oldest event is discarded.
        """
        clock_dio = digitalio.DigitalInOut(clock)
        clock_dio.switch_to_output(
            value=False, drive_mode=digitalio.DriveMode.PUSH_PULL
        )
        self._clock = clock_dio

        data_dio = digitalio.DigitalInOut(data)
        data_dio.switch_to_input()
        self._data = data_dio

        latch_dio = digitalio.DigitalInOut(latch)
        latch_dio.switch_to_output(value=True, drive_mode=digitalio.DriveMode.PUSH_PULL)
        self._latch = latch_dio
        self._value_to_latch = value_to_latch

        self._currently_pressed = [False] * key_count
        self._previously_pressed = [False] * key_count
        self._value_when_pressed = value_when_pressed
        self._key_count = key_count

        super().__init__(interval, max_events, self._keypad_shiftregisterkeys_scan)

    def deinit(self):
        """Stop scanning and release the pins."""
        super().deinit()
        self._clock.deinit()
        self._data.deinit()
        self._latch.deinit()

    def reset(self):
        """
        Reset the internal state of the scanner to assume that all keys are now released.
        Any key that is already pressed at the time of this call will therefore immediately cause
        a new key-pressed event to occur.
        """
        self._currently_pressed = self._previously_pressed = [False] * self._key_count

    @property
    def key_count(self):
        """The number of keys that are being scanned. (read-only)"""
        return self._key_count

    @property
    def events(self):
        """The ``EventQueue`` associated with this `Keys` object. (read-only)"""
        return self._events

    def _keypad_shiftregisterkeys_scan(self):
        self._latch.value = self._value_to_latch
        for key_number in range(self._key_count):
            self._clock.value = False
            self._previously_pressed[key_number] = self._currently_pressed[key_number]
            current = self._data.value == self._value_when_pressed
            self._currently_pressed[key_number] = current
            self._clock.value = True
            if self._previously_pressed[key_number] != current:
                self._events.keypad_eventqueue_record(key_number, current)

        self._latch.value = not self._value_to_latch



"""
`analogio` - Analog input and output control
=================================================
See `CircuitPython:analogio` in CircuitPython for more details.
* Author(s): Carter Nelson
"""
from adafruit_blinka import ContextManaged
from .rp2040_u2if import rp2040_u2if


class AnalogIn(ContextManaged):
    """AnalogIn Base Class for RP2040 u2if"""

    def __init__(self, pin):
        self.pin_id = pin.id
        rp2040_u2if.adc_init_pin(self.pin_id)

    @property
    def value(self):
        """Read the ADC and return the value"""
        return rp2040_u2if.adc_get_value(self.pin_id) << 4

    # pylint: disable=no-self-use
    @value.setter
    def value(self, value):
        # emulate what CircuitPython does
        raise AttributeError("'AnalogIn' object has no attribute 'value'")

    # pylint: enable=no-self-use

    def deinit(self):
        pass


class AnalogIn_Pico(AnalogIn):
    """AnalogIn Base Class for Pico u2if"""

    def __init__(self, pin):
        # per their pinout, why only two?
        if pin.id not in (26, 27):
            raise ValueError("Pin does not support ADC.")
        super().__init__(pin)


class AnalogIn_Feather(AnalogIn):
    """AnalogIn Base Class for Feather u2if"""

    def __init__(self, pin):
        if pin.id not in (26, 27, 28):
            raise ValueError("Pin does not support ADC.")
        super().__init__(pin)


class AnalogIn_QTPY(AnalogIn):
    """AnalogIn Base Class for QT Py 2040 u2if"""

    def __init__(self, pin):
        if pin.id not in (26, 27, 28):
            raise ValueError("Pin does not support ADC.")
        super().__init__(pin)


class AnalogIn_ItsyBitsy(AnalogIn):
    """AnalogIn Base Class for ItsyBitsy 2040 u2if"""

    def __init__(self, pin):
        if pin.id not in (26, 27, 28):
            raise ValueError("Pin does not support ADC.")
        super().__init__(pin)



"""A Pin class for use with Rockchip RK3399."""

from adafruit_blinka.microcontroller.generic_linux.sysfs_pin import Pin

GPIO1_A7 = Pin(39)
GPIO1_B0 = Pin(40)
GPIO1_B1 = Pin(41)
GPIO1_B2 = Pin(42)
GPIO2_A0 = Pin(64)
GPIO2_A1 = Pin(65)
GPIO2_A7 = Pin(71)
GPIO2_B0 = Pin(72)
GPIO2_B1 = Pin(73)
GPIO2_B2 = Pin(74)
GPIO2_B3 = Pin(75)
GPIO2_B4 = Pin(76)
GPIO3_C0 = Pin(112)
GPIO4_A3 = Pin(131)
GPIO4_A4 = Pin(132)
GPIO4_A5 = Pin(133)
GPIO4_A6 = Pin(134)
GPIO4_A7 = Pin(135)
GPIO4_C2 = Pin(146)
GPIO4_C3 = Pin(147)
GPIO4_C4 = Pin(148)
GPIO4_C5 = Pin(149)
GPIO4_C6 = Pin(150)
GPIO4_D2 = Pin(154)
GPIO4_D4 = Pin(156)
GPIO4_D5 = Pin(157)
GPIO4_D6 = Pin(158)
ADC_IN0 = 1

# I2C
I2C2_SDA = GPIO2_A0
I2C2_SCL = GPIO2_A1
I2C6_SDA = GPIO2_B1
I2C6_SCL = GPIO2_B2
I2C7_SDA = GPIO2_A7
I2C7_SCL = GPIO2_B0

# SPI
SPI1_CS = GPIO1_B2
SPI1_SCLK = GPIO1_B1
SPI1_MISO = GPIO1_B0
SPI1_MOSI = GPIO1_A7
SPI2_CS = GPIO2_B4
SPI2_SCLK = GPIO2_A1
SPI2_MISO = GPIO2_B1
SPI2_MOSI = GPIO2_B2

# UART
UART0_TX = GPIO4_C4
UART0_RX = GPIO4_C3

# PWM
PWM0 = GPIO4_C2
PWM1 = GPIO4_C6

# ordered as i2cId, SCL, SDA
i2cPorts = (
    (0, I2C2_SCL, I2C2_SDA),
    (1, I2C6_SCL, I2C6_SDA),
    (7, I2C7_SCL, I2C7_SDA),
)

# ordered as spiId, sckId, mosiId, misoId
spiPorts = ((1, SPI1_SCLK, SPI1_MOSI, SPI1_MISO),)

# SysFS pwm outputs, pwm channel and pin in first tuple
pwmOuts = (
    ((0, 0), PWM0),
    ((1, 0), PWM1),
)

# SysFS analog inputs, Ordered as analog analogInId, device, and channel
analogIns = ((ADC_IN0, 0, 0),)



"""
Pin definitions for the QT Py RP2040 with u2if firmware.

Adafruit CircuitPython 6.2.0 on 2021-04-05; Adafruit QTPy RP2040 with rp2040
>>> import board
>>> board.
A0              A1              A2              A3
BUTTON          D0              D1              D10
D2              D3              D4              D5
D6              D7              D8              D9
I2C             MISO            MOSI            NEOPIXEL
NEOPIXEL_POWER  RX              SCK             SCL
SCL1            SDA             SDA1            SPI
TX              UART
"""


from adafruit_blinka.microcontroller.rp2040_u2if import pin

D0 = pin.GP29
D1 = pin.GP28
D2 = pin.GP27
D3 = pin.GP26
D4 = pin.GP24
D5 = pin.GP25
D6 = pin.GP20
D7 = pin.GP5
D8 = pin.GP6
D9 = pin.GP4
D10 = pin.GP3

# A0 = pin.GP29 # not currently supported in firmware
A1 = pin.GP28
A2 = pin.GP27
A3 = pin.GP26

SCL = pin.GP25
SDA = pin.GP24

SCL1 = pin.GP23
SDA1 = pin.GP22

SCLK = SCK = pin.GP6
MOSI = pin.GP3
MISO = pin.GP4

NEOPIXEL = pin.GP12
NEOPIXEL_POWER = pin.GP11

BUTTON = pin.GP21

# access u2if via pin instance to open for specifc VID/PID
# pylint:disable = protected-access
pin.GP0._u2if_open_hid(0x239A, 0x00F7)



"""I2C Classes for RP2040s with u2if firmware"""
from .rp2040_u2if import rp2040_u2if


class I2C:
    """I2C Base Class for RP2040 u2if"""

    def __init__(self, index, *, frequency=100000):
        self._index = index
        rp2040_u2if.i2c_set_port(self._index)
        rp2040_u2if.i2c_configure(frequency)

    def scan(self):
        """Perform an I2C Device Scan"""
        rp2040_u2if.i2c_set_port(self._index)
        return rp2040_u2if.i2c_scan()

    # pylint: disable=unused-argument
    def writeto(self, address, buffer, *, start=0, end=None, stop=True):
        """Write data from the buffer to an address"""
        rp2040_u2if.i2c_set_port(self._index)
        rp2040_u2if.i2c_writeto(address, buffer, start=start, end=end)

    def readfrom_into(self, address, buffer, *, start=0, end=None, stop=True):
        """Read data from an address and into the buffer"""
        rp2040_u2if.i2c_set_port(self._index)
        rp2040_u2if.i2c_readfrom_into(address, buffer, start=start, end=end)

    def writeto_then_readfrom(
        self,
        address,
        buffer_out,
        buffer_in,
        *,
        out_start=0,
        out_end=None,
        in_start=0,
        in_end=None,
        stop=False,
    ):
        """Write data from buffer_out to an address and then
        read data from an address and into buffer_in
        """
        rp2040_u2if.i2c_set_port(self._index)
        rp2040_u2if.i2c_writeto_then_readfrom(
            address,
            buffer_out,
            buffer_in,
            out_start=out_start,
            out_end=out_end,
            in_start=in_start,
            in_end=in_end,
        )

    # pylint: enable=unused-argument


class I2C_Pico(I2C):
    """I2C Class for Pico u2if"""

    def __init__(self, scl, sda, *, frequency=100000):
        index = None
        if scl.id == 5 and sda.id == 4:
            index = 0
        if scl.id == 15 and sda.id == 14:
            index = 1
        if index is None:
            raise ValueError("I2C not found on specified pins.")
        self._index = index

        super().__init__(index, frequency=frequency)


class I2C_Feather(I2C):
    """I2C Class for Feather u2if"""

    def __init__(self, scl, sda, *, frequency=100000):
        index = None
        if scl.id == 3 and sda.id == 2:
            index = 1
        if index is None:
            raise ValueError("I2C not found on specified pins.")
        self._index = index

        super().__init__(index, frequency=frequency)


class I2C_QTPY(I2C):
    """I2C Class for QT Py 2if"""

    def __init__(self, scl, sda, *, frequency=100000):
        index = None
        if scl.id == 25 and sda.id == 24:
            index = 0
        if scl.id == 23 and sda.id == 22:
            index = 1
        if index is None:
            raise ValueError("I2C not found on specified pins.")
        self._index = index

        super().__init__(index, frequency=frequency)


class I2C_ItsyBitsy(I2C):
    """I2C Class for ItsyBitsy u2if"""

    def __init__(self, scl, sda, *, frequency=100000):
        index = None
        if scl.id == 3 and sda.id == 2:
            index = 1
        if index is None:
            raise ValueError("I2C not found on specified pins.")
        self._index = index

        super().__init__(index, frequency=frequency)


class I2C_MacroPad(I2C):
    """I2C Class for MacroPad u2if"""

    def __init__(self, scl, sda, *, frequency=100000):
        index = None
        if scl.id == 21 and sda.id == 20:
            index = 0
        if index is None:
            raise ValueError("I2C not found on specified pins.")
        self._index = index
        super().__init__(index, frequency=frequency)


class I2C_QT2040_Trinkey(I2C):
    """I2C Class for QT2040 Trinkey u2if"""

    def __init__(self, scl, sda, *, frequency=100000):
        index = None
        if scl.id == 17 and sda.id == 16:
            index = 0
        if index is None:
            raise ValueError("I2C not found on specified pins.")
        self._index = index

        super().__init__(index, frequency=frequency)



"""FT2232H pin names"""

from adafruit_blinka.microcontroller.ftdi_mpsse.mpsse.pin import Pin

# See https://eblot.github.io/pyftdi/pinout.html for detailed FTDI device pinout information

# MPSSE Port A
AD4 = Pin(4, interface_id=0)
AD5 = Pin(5, interface_id=0)
AD6 = Pin(6, interface_id=0)
AD7 = Pin(7, interface_id=0)
AC0 = Pin(8, interface_id=0)
AC1 = Pin(9, interface_id=0)
AC2 = Pin(10, interface_id=0)
AC3 = Pin(11, interface_id=0)
AC4 = Pin(12, interface_id=0)
AC5 = Pin(13, interface_id=0)
AC6 = Pin(14, interface_id=0)
AC7 = Pin(15, interface_id=0)

SCL0 = Pin(interface_id=0)
SDA0 = Pin(interface_id=0)
SCK0 = SCLK0 = Pin(interface_id=0)
MOSI0 = Pin(interface_id=0)
MISO0 = Pin(interface_id=0)

# MPSSE Port B
BD4 = Pin(4, interface_id=1)
BD5 = Pin(5, interface_id=1)
BD6 = Pin(6, interface_id=1)
BD7 = Pin(7, interface_id=1)
BC0 = Pin(8, interface_id=1)
BC1 = Pin(9, interface_id=1)
BC2 = Pin(10, interface_id=1)
BC3 = Pin(11, interface_id=1)
BC4 = Pin(12, interface_id=1)
BC5 = Pin(13, interface_id=1)
BC6 = Pin(14, interface_id=1)
BC7 = Pin(15, interface_id=1)

SCL1 = Pin(interface_id=1)
SDA1 = Pin(interface_id=1)
SCK1 = SCLK1 = Pin(interface_id=1)
MOSI1 = Pin(interface_id=1)
MISO1 = Pin(interface_id=1)

i2cPorts = (
    (0, SCL0, SDA0),
    (1, SCL1, SDA1),
)

spiPorts = (
    (0, SCLK0, MOSI0, MISO0),
    (1, SCLK1, MOSI1, MISO1),
)



"""A Pin class for use with StarFive JH71x0."""

from adafruit_blinka.microcontroller.generic_linux.libgpiod_pin import Pin

D0 = Pin(9)
D1 = Pin(10)
D4 = Pin(46)
D5 = Pin(8)
D6 = Pin(6)
D7 = Pin(11)
D8 = Pin(15)
D9 = Pin(16)
D10 = Pin(18)
D11 = Pin(12)
D12 = Pin(7)
D13 = Pin(5)
D14 = Pin(14)
D15 = Pin(13)
D16 = Pin(4)
D17 = Pin(44)
D18 = Pin(45)
D19 = Pin(3)
D20 = Pin(2)
D21 = Pin(0)
D22 = Pin(20)
D23 = Pin(21)
D24 = Pin(19)
D25 = Pin(17)
D26 = Pin(1)
D27 = Pin(22)

# I2C
I2C1_SDA = Pin(48)
I2C1_SCL = Pin(47)
I2C2_SDA = Pin(59)
I2C2_SCL = Pin(60)
I2C3_SDA = Pin(61)
I2C3_SCL = Pin(62)

# SPI
SPI_MISO = D9
SPI_MOSI = D10
SPI_SCLK = D11

# UART
UART_TX = D14
UART_RX = D15

# ordered as i2cId, SCL, SDA
i2cPorts = (
    (0, I2C1_SCL, I2C1_SDA),
    (1, I2C2_SCL, I2C2_SDA),
    (2, I2C3_SCL, I2C3_SDA),
)

# ordered as spiId, sckId, mosiId, misoId
spiPorts = ((0, SPI_SCLK, SPI_MOSI, SPI_MISO),)



"""
`rainbowio` - Provides the `colorwheel()` function
===========================================================
See `CircuitPython:rainbowio` in CircuitPython for more details.
Not supported by all boards.

* Author(s): Kattni Rembor, Carter Nelson
"""


def colorwheel(color_value):
    """
    A colorwheel. ``0`` and ``255`` are red, ``85`` is green, and ``170`` is blue, with the values
    between being the rest of the rainbow.

    :param int color_value: 0-255 of color value to return
    :return: tuple of RGB values
    """
    color_value = int(color_value)
    if color_value < 0 or color_value > 255:
        r = 0
        g = 0
        b = 0
    elif color_value < 85:
        r = int(255 - color_value * 3)
        g = int(color_value * 3)
        b = 0
    elif color_value < 170:
        color_value -= 85
        r = 0
        g = int(255 - color_value * 3)
        b = int(color_value * 3)
    else:
        color_value -= 170
        r = int(color_value * 3)
        g = 0
        b = int(255 - color_value * 3)
    return r << 16 | g << 8 | b



"""
`microcontroller` - Pin references and cpu functionality
========================================================

* Author(s): Melissa LeBlanc-Williams
"""

import sys
import time

from adafruit_platformdetect.constants import chips as ap_chip
from adafruit_blinka import Enum
from adafruit_blinka.agnostic import board_id, chip_id


def delay_us(delay):
    """Sleep for delay usecs."""
    time.sleep(delay / 1e6)


class Pin(Enum):
    """
    Identifies an IO pin on the microcontroller.

    They are fixed by the hardware so they cannot be constructed on demand. Instead, use board or
    microcontroller.pin to reference the desired pin.
    """

    def __init__(self, pin_id):
        """Identifier for pin, referencing platform-specific pin id"""
        self.id = pin_id

    def __repr__(self):
        # pylint: disable=import-outside-toplevel, cyclic-import
        import board

        for key in dir(board):
            if getattr(board, key) is self:
                return "board.{}".format(key)
        import microcontroller.pin as pin

        # pylint: enable=import-outside-toplevel, cyclic-import

        for key in dir(pin):
            if getattr(pin, key) is self:
                return "microcontroller.pin.{}".format(key)
        return repr(self)


# We intentionally are patching into this namespace so skip the wildcard check.
# pylint: disable=unused-wildcard-import,wildcard-import,ungrouped-imports

if chip_id == ap_chip.ESP8266:
    from adafruit_blinka.microcontroller.esp8266 import *
elif chip_id == ap_chip.STM32F405:
    from adafruit_blinka.microcontroller.stm32.stm32f405 import *
elif chip_id == ap_chip.RP2040:
    from adafruit_blinka.microcontroller.rp2040 import *
elif chip_id == ap_chip.BCM2XXX:
    if board_id in [
        "RASPBERRY_PI_4B",
        "RASPBERRY_PI_400",
        "RASPBERRY_PI_CM4",
    ]:
        from adafruit_blinka.microcontroller.bcm2711.pin import *
    else:
        from adafruit_blinka.microcontroller.bcm283x.pin import *
elif chip_id == ap_chip.DRA74X:
    from adafruit_blinka.microcontroller.dra74x.pin import *
elif chip_id == ap_chip.AM33XX:
    from adafruit_blinka.microcontroller.am335x import *
elif chip_id == ap_chip.JH71x0:
    from adafruit_blinka.microcontroller.starfive.JH71x0 import *
elif chip_id == ap_chip.SUN8I:
    from adafruit_blinka.microcontroller.allwinner.h3 import *
elif chip_id == ap_chip.H5:
    from adafruit_blinka.microcontroller.allwinner.h5.pin import *
elif chip_id == ap_chip.H6:
    from adafruit_blinka.microcontroller.allwinner.h6.pin import *
elif chip_id == ap_chip.H616:
    from adafruit_blinka.microcontroller.allwinner.h616.pin import *
elif chip_id == ap_chip.SAMA5:
    from adafruit_blinka.microcontroller.sama5 import *
elif chip_id == ap_chip.T210:
    from adafruit_blinka.microcontroller.tegra.t210 import *
elif chip_id == ap_chip.T186:
    from adafruit_blinka.microcontroller.tegra.t186 import *
elif chip_id == ap_chip.T194:
    from adafruit_blinka.microcontroller.tegra.t194 import *
elif chip_id == ap_chip.S905:
    from adafruit_blinka.microcontroller.amlogic.s905.pin import *
elif chip_id == ap_chip.S905X3:
    from adafruit_blinka.microcontroller.amlogic.s905x3.pin import *
elif chip_id == ap_chip.S922X:
    from adafruit_blinka.microcontroller.amlogic.s922x.pin import *
elif chip_id == ap_chip.EXYNOS5422:
    from adafruit_blinka.microcontroller.samsung.exynos5422.pin import *
elif chip_id == ap_chip.APQ8016:
    from adafruit_blinka.microcontroller.snapdragon.apq8016.pin import *
elif chip_id == ap_chip.A64:
    from adafruit_blinka.microcontroller.allwinner.a64.pin import *
elif chip_id == ap_chip.A33:
    from adafruit_blinka.microcontroller.allwinner.a33.pin import *
elif chip_id == ap_chip.RK3308:
    from adafruit_blinka.microcontroller.rockchip.rk3308.pin import *
elif chip_id == ap_chip.RK3399:
    from adafruit_blinka.microcontroller.rockchip.rk3399.pin import *
elif chip_id == ap_chip.RK3328:
    from adafruit_blinka.microcontroller.rockchip.rk3328.pin import *
elif chip_id == ap_chip.H3:
    from adafruit_blinka.microcontroller.allwinner.h3.pin import *
elif chip_id == ap_chip.H5:
    from adafruit_blinka.microcontroller.allwinner.h5.pin import *
elif chip_id == ap_chip.IMX8MX:
    from adafruit_blinka.microcontroller.nxp_imx8m import *
elif chip_id == ap_chip.IMX6ULL:
    from adafruit_blinka.microcontroller.nxp_imx6ull import *
elif chip_id == ap_chip.HFU540:
    from adafruit_blinka.microcontroller.hfu540.pin import *
elif chip_id == ap_chip.BINHO:
    from adafruit_blinka.microcontroller.nova import *
elif chip_id == ap_chip.LPC4330:
    from adafruit_blinka.microcontroller.nxp_lpc4330 import *
elif chip_id == ap_chip.MCP2221:
    from adafruit_blinka.microcontroller.mcp2221.pin import *
elif chip_id == ap_chip.MIPS24KC:
    from adafruit_blinka.microcontroller.atheros.ar9331 import *
elif chip_id == ap_chip.MIPS24KEC:
    from adafruit_blinka.microcontroller.mips24kec import *
elif chip_id == ap_chip.FT232H:
    from adafruit_blinka.microcontroller.ftdi_mpsse.ft232h.pin import *
elif chip_id == ap_chip.FT2232H:
    from adafruit_blinka.microcontroller.ftdi_mpsse.ft2232h.pin import *
elif chip_id == ap_chip.PENTIUM_N3710:
    from adafruit_blinka.microcontroller.pentium.n3710.pin import *
elif chip_id == ap_chip.STM32MP157:
    from adafruit_blinka.microcontroller.stm32.stm32mp157.pin import *
elif chip_id == ap_chip.MT8167:
    from adafruit_blinka.microcontroller.mt8167.pin import *
elif chip_id == ap_chip.RP2040_U2IF:
    from adafruit_blinka.microcontroller.rp2040_u2if.pin import *
elif chip_id == ap_chip.GENERIC_X86:
    print("WARNING: GENERIC_X86 is not fully supported. Some features may not work.")
elif "sphinx" in sys.modules:
    pass
else:
    raise NotImplementedError("Microcontroller not supported:", chip_id)



"""PWMOut Class for RP2040s with u2if"""
from .rp2040_u2if import rp2040_u2if


class PWMOut:
    """Base Pulse Width Modulation Output Class"""

    def __init__(self, pin, *, frequency=500, duty_cycle=0, variable_frequency=False):
        rp2040_u2if.pwm_configure(
            pin,
            frequency=frequency,
            duty_cycle=duty_cycle,
            variable_frequency=variable_frequency,
        )

        self._pin = pin

    def __del__(self):
        self.deinit()

    def __enter__(self):
        return self

    def __exit__(self, t, value, traceback):
        self.deinit()

    def deinit(self):
        """Deinit PWM."""
        rp2040_u2if.pwm_deinit(self._pin)

    @property
    def duty_cycle(self):
        """The PWM's output duty cycle, 16-bit."""
        return rp2040_u2if.pwm_get_duty_cycle(self._pin)

    @duty_cycle.setter
    def duty_cycle(self, duty_cycle):
        rp2040_u2if.pwm_set_duty_cycle(self._pin, duty_cycle)

    @property
    def frequency(self):
        """The PWM's output frequency in Hertz."""
        return rp2040_u2if.pwm_get_frequency(self._pin)

    @frequency.setter
    def frequency(self, frequency):
        rp2040_u2if.pwm_set_frequency(self._pin, frequency)



"""
Pin definitions for the MacroPad RP2040 with u2if firmware.

Adafruit CircuitPython 6.2.0 on 2021-04-05; Adafruit MacroPad RP2040 with rp2040
>>> import board
>>> board.
KEY1            KEY2            KEY3            KEY4
KEY5            KEY6            KEY7            KEY8
KEY9            KEY10           KEY11           KEY12
LED             SPEAKER_ENABLE  SPEAKER         ENCODER_SWITCH
BUTTON          ENCODER_A       ROTA            ENCODER_B
ROTB            NEOPIXEL        SDA             SCL
OLED_CS         OLED_RESET      OLED_DC         SCLK
SCK             MOSI            MISO            I2C
SPI             UART
"""

from adafruit_blinka.microcontroller.rp2040_u2if import pin

KEY1 = pin.GP1
KEY2 = pin.GP2
KEY3 = pin.GP3
KEY4 = pin.GP4
KEY5 = pin.GP5
KEY6 = pin.GP6
KEY7 = pin.GP7
KEY8 = pin.GP8
KEY9 = pin.GP9
KEY10 = pin.GP10
KEY11 = pin.GP11
KEY12 = pin.GP12

LED = pin.GP13

SPEAKER_ENABLE = pin.GP4
SPEAKER = pin.GP16

ENCODER_SWITCH = BUTTON = pin.GP0

ENCODER_A = ROTA = pin.GP17
ENCODER_B = ROTB = pin.GP18

NEOPIXEL = pin.GP19

SDA = pin.GP20
SCL = pin.GP21

OLED_CS = pin.GP22
OLED_RESET = pin.GP23
OLED_DC = pin.GP24

SCLK = SCK = pin.GP26
MOSI = pin.GP27
MISO = pin.GP28

# access u2if via pin instance to open for specifc VID/PID
# pylint:disable = protected-access
pin.GP0._u2if_open_hid(0x239A, 0x0107)



"""
Support for getting the URL from the BLINKA_FT232H
and BLINKA_FT2232H_{} environment variables.
"""

import os


def get_ft232h_url():
    """
    Return the FTDI url to use. If BLINKA_FT232H starts with ftdi:, returns
    that. Otherwise, returns a default value.
    """

    url = os.environ.get("BLINKA_FT232H", "1")

    if url.startswith("ftdi:"):
        return url

    return "ftdi://ftdi:ft232h/1"


def get_ft2232h_url(interface_id):
    """
    Return the FTDI url to use. If BLINKA_FT2232H_{} starts with ftdi:, returns
    that. Otherwise, returns a default value.
    """

    url = os.environ.get("BLINKA_FT2232H_{}".format(interface_id), "1")

    if url.startswith("ftdi:"):
        return url

    return "ftdi://ftdi:ft2232h/{}".format(interface_id + 1)



"""
`pulseio` - Pulse Width Modulation input and output control
===========================================================
See `CircuitPython:pulseio` in CircuitPython for more details.
Not supported by all boards.

* Author(s): Melissa LeBlanc-Williams
"""

import sys

from adafruit_blinka.agnostic import detector

# pylint: disable=unused-import

if detector.board.any_raspberry_pi:
    from adafruit_blinka.microcontroller.bcm283x.pulseio.PulseIn import PulseIn
elif "sphinx" in sys.modules:
    pass
else:
    raise NotImplementedError("pulseio not supported for this board.")



"""FT232H pin names"""

from adafruit_blinka.microcontroller.ftdi_mpsse.mpsse.pin import Pin

# See https://eblot.github.io/pyftdi/pinout.html for detailed FTDI device pinout information

# MPSSE Port A
D4 = Pin(4)
D5 = Pin(5)
D6 = Pin(6)
D7 = Pin(7)
C0 = Pin(8)
C1 = Pin(9)
C2 = Pin(10)
C3 = Pin(11)
C4 = Pin(12)
C5 = Pin(13)
C6 = Pin(14)
C7 = Pin(15)

SCL = Pin()
SDA = Pin()
SCK = SCLK = Pin()
MOSI = Pin()
MISO = Pin()



"""SPI Classes for RP2040s with u2if firmware"""
from .rp2040_u2if import rp2040_u2if

# pylint: disable=protected-access, no-self-use
class SPI:
    """SPI Base Class for RP2040 u2if"""

    MSB = 0

    def __init__(self, index, *, baudrate=100000):
        self._index = index
        self._frequency = baudrate
        rp2040_u2if.spi_set_port(self._index)
        rp2040_u2if.spi_configure(self._frequency)

    # pylint: disable=too-many-arguments,unused-argument
    def init(
        self,
        baudrate=1000000,
        polarity=0,
        phase=0,
        bits=8,
        firstbit=MSB,
        sck=None,
        mosi=None,
        miso=None,
    ):
        """Initialize the Port"""
        self._frequency = baudrate
        rp2040_u2if.spi_set_port(self._index)
        rp2040_u2if.spi_configure(self._frequency)

    # pylint: enable=too-many-arguments

    @property
    def frequency(self):
        """Return the current frequency"""
        return self._frequency

    def write(self, buf, start=0, end=None):
        """Write data from the buffer to SPI"""
        rp2040_u2if.spi_write(buf, start=start, end=end)

    def readinto(self, buf, start=0, end=None, write_value=0):
        """Read data from SPI and into the buffer"""
        rp2040_u2if.spi_readinto(buf, start=start, end=end, write_value=write_value)

    # pylint: disable=too-many-arguments
    def write_readinto(
        self, buffer_out, buffer_in, out_start=0, out_end=None, in_start=0, in_end=None
    ):
        """Perform a half-duplex write from buffer_out and then
        read data into buffer_in
        """
        rp2040_u2if.spi_write_readinto(
            buffer_out,
            buffer_in,
            out_start=out_start,
            out_end=out_end,
            in_start=in_start,
            in_end=in_end,
        )

    # pylint: enable=too-many-arguments


class SPI_Pico(SPI):
    """SPI Class for Pico u2if"""

    def __init__(self, clock, *, baudrate=100000):
        index = None
        if clock.id == 18:
            index = 0
        if clock.id == 10:
            index = 1
        if index is None:
            raise ValueError("No SPI port on specified pin.")
        super().__init__(index, baudrate=baudrate)


class SPI_Feather(SPI):
    """SPI Class for Feather u2if"""

    def __init__(self, clock, *, baudrate=100000):
        index = None
        if clock.id == 18:
            index = 0
        if index is None:
            raise ValueError("No SPI port on specified pin.")
        super().__init__(index, baudrate=baudrate)


class SPI_QTPY(SPI):
    """SPI Class for QT Py u2if"""

    def __init__(self, clock, *, baudrate=100000):
        index = None
        if clock.id == 6:
            index = 0
        if index is None:
            raise ValueError("No SPI port on specified pin.")
        super().__init__(index, baudrate=baudrate)


class SPI_ItsyBitsy(SPI):
    """SPI Class for ItsyBitsy u2if"""

    def __init__(self, clock, *, baudrate=100000):
        index = None
        if clock.id == 18:
            index = 0
        if index is None:
            raise ValueError("No SPI port on specified pin.")
        super().__init__(index, baudrate=baudrate)


class SPI_MacroPad(SPI):
    """SPI Class for MacroPad u2if"""

    def __init__(self, clock, *, baudrate=100000):
        index = None
        if clock.id == 26:
            index = 1
        if index is None:
            raise ValueError("No SPI port on specified pin.")
        super().__init__(index, baudrate=baudrate)



"""Pin definitions for the LubanCat STM32MP157."""

from adafruit_blinka.microcontroller.stm32.stm32mp157 import pin

# Pro board pin header J1 named GPIO_PAx, pin header J2 named GPIO_PBx

# Board pin name [= alias] = RPI name [= alias] = pin name

# connector J1
GPIO_PA3 = USART1_CTS = PZ3 = pin.PZ3
GPIO_PA4 = USART1_RTS = PZ5 = pin.PZ5
GPIO_PA5 = USART1_TX = PZ7 = pin.PZ7
GPIO_PA6 = USART1_RX = PZ6 = pin.PZ6
GPIO_PA7 = USART3_TX = PB10 = pin.PB10
GPIO_PA8 = USART3_RX = PB12 = pin.PB12
GPIO_PA11 = FDCAN1_TX = PA12 = pin.PA12
GPIO_PA12 = FDCAN1_RX = PA11 = pin.PA11
# connector J2
GPIO_PB7 = UART4_TX = PG11 = pin.PG11
GPIO_PB8 = UART4_RX = PB2 = pin.PB2
GPIO_PB11 = QSPI_IO0 = PF8 = pin.PF8
GPIO_PB12 = QSPI_IO1 = PF9 = pin.PF9
GPIO_PB13 = QSPI_IO2 = PF7 = pin.PF7
GPIO_PB14 = QSPI_IO3 = PF6 = pin.PF6
GPIO_PB15 = QSPI_CLK = PF10 = pin.PF10
GPIO_PB16 = QSPI_NCS = PB6 = pin.PB6

# general gpio as LED、KEY function
# LED
LED_RED = PA13 = pin.PA13
LED_GREEN = PB5 = pin.PG2
LED_BLUE = PB5 = pin.PB5
# KEY
KEY1 = PB13 = pin.PB13
KEY2 = PH7 = pin.PH7
# BEEP
BEEP = PC13 = pin.PC13

# general gpio as I2C function
# I2C1
GPIO_PA13 = I2C1_SCL = SCL1 = SCL = pin.PF14
GPIO_PA14 = I2C1_SDA = SDA1 = SDA = pin.PF15
# I2C2
GPIO_PA15 = I2C2_SCL = SCL2 = pin.PZ0
GPIO_PA16 = I2C2_SDA = SDA2 = pin.PZ1

# general gpio as analog input function
GPIO_PB3 = ADC_IN0 = ANA0 = A0 = pin.PAN0
GPIO_PB4 = ADC_IN1 = ANA1 = A1 = pin.PAN1
